package ShortestPath;

import java.util.HashMap;

public class Heap {
	Node[] data; 	// heap data, actually a tree
	int used;		// hash map for quick finding index of a node
	
	HashMap <Node, Integer> map;
	
	public Heap () {
		data = new Node[16];
		map = new HashMap<Node, Integer>();
		used = 0;
	}
	
	private void Grow(){
		Node[] newData = new Node[data.length*2];
		for (int i=0; i<data.length; ++i)
			newData[i] = data[i];
		data = newData;
	}
	
	// set value at index
	// keeps both data and map up to date
	private void Set(int index, Node value)
	{
		data[index] = value;
		map.put(value, index);
	}
	
	private boolean Smaller(Node l, Node r)
	{
		return l.GetPathLen() < r.GetPathLen();
	}

	public void Insert(Node n)
	{
		if (used == data.length) 
			Grow();
		int current = used;
		int parent = (current-1)/2;
		while (current > 0 && Smaller(n, data[parent]))
		{
			Set(current, data[parent]);
			current = parent;
			parent = (current-1)/2;
		}
		Set(current, n);
		map.put(n, current);
		used++;
	}
	

	// return child with smaller value. If larger child index (2*current+2) is is not in range, return smaller child index
	private int BestChild(int current)
	{
		if (2*current+2 >= used || Smaller(data[2*current+1] , data[2*current+2])) // take left branch 
			return 2*current+1; 
		else // right branch 
			return 2*current+2;
	}
	
	public Node ExtractRoot()
	{
		assert(used > 0);
		Node min = data[0];
		Node value = data[used-1];
		int current= 0;
		int next = BestChild(current);
		while(next < used && !Smaller(value, data[next])) {
			Set(current, data[next]);
			current=next;
			next = BestChild(current);
		}
		Set(current, value);
		map.remove(min);
		used--;
		return min;
	}	
	
	public void DecreaseKey(Node n)
	{
		int current = map.get(n);
		int parent = (current-1)/2;
		while (current > 0 && Smaller(n, data[parent]))
		{
			Set(current, data[parent]);
			current = parent;
			parent = (current-1)/2;
		}
		Set(current, n);
	}

}

